import {BadRequestError, CustomError, ForbiddenError, InternalServerError, Logger} from '@ztimson/utils';
import {Response, NextFunction} from 'express';
import {environment} from './environment';

export class PermissionsError extends ForbiddenError {
	constructor(message?: string) {
		super(message.replace(/^.+?:/g, 'Permissions required:'));
	}
}

export class ValidationError extends BadRequestError {
	constructor(message: string = 'Unknown') {
		super(`Validation Error: ${message}`);
	}

	static from(err: any) {
		const f = err.fields;
		const k = Object.keys(f);
		return new ValidationError(f[k[0]].message);
	}
}

export const errorHandler = fn => (req: any, res: Response, next: NextFunction) => {
	Promise.resolve(fn(req, res, next)).catch(next);
};

const logger = new Logger();
export const errorMiddleware = (err: Error | CustomError, req?: any, resp?: Response, next?: NextFunction) => {
	const e: CustomError = CustomError.instanceof(err) ? <CustomError>err :
		/Requires (one of|all):/g.test(err.message) ? PermissionsError.from(err) :
			err?.stack?.includes('SqliteError') ? BadRequestError.from(err) :
			err?.stack?.includes('ValidateError') ? ValidationError.from(err) :
			InternalServerError.from(err);

	const clientError = e.code >= 400 && e.code < 500;
	if(req) logger[clientError ? (environment.production ? 'debug' : 'warn') : 'error'](`[${req.ipV4}] ${req.method} ${req.url ? decodeURI(req.url) : null} -> ${e.code}`);
	if(!clientError || !environment.production) {
		const message = (e.stack || `${e.message} (No Stack Trace)`);
		logger[clientError ? 'warn' : 'error'](message);
	}

	if(resp) {
		resp['logged'] = true;
		resp.statusMessage = e.message.replaceAll('\n', ' ');
		resp.contentType('application/json').status(e.code).send({
			method: req?.method,
			url: req?.url,
			code: e.code,
			timestamp: (new Date()).toISOString(),
			message: e.message,
			stack: environment.production ? undefined : e.stack,
		});
	}
}
