import compression from 'compression';
import cors from 'cors';
import express from 'express';
import http from 'node:http';
import process from 'node:process';
import {environment} from './environment';
import {errorHandler, errorMiddleware} from './error.middleware';
import {loggerMiddleware} from './logger.middleware';

if(!environment.token) throw new Error('Please provide a Finnhub token');
if(!environment.tickers.length) throw new Error('Please provide a CSV list of tickers');

const app = express();
const httpServer: http.Server = http.createServer(app);
app.use(cors({origin: '*'}))
app.use(compression());
app.use(loggerMiddleware);

// Graceful shutdown
let shuttingDown = false;
['SIGHUP', 'SIGINT', 'SIGTERM'].forEach((signal) => {
	process.on(signal, (signal, value) => {
		if(shuttingDown) return; // Already shutting down
		shuttingDown = true;
		console.warn(`${signal} Received, shutting down...`);
		httpServer.close(() => process.exit(128 + value));
	});
});

app.get('/', errorHandler(async (req, res) => {
	const results = await Promise.all(environment.tickers.map(ticker =>
		fetch(`https://finnhub.io/api/v1/quote?symbol=${ticker}&token=${environment.token}`)
			.then(resp => resp.json())));

	let metrics = '# HELP stock_price Current stock price\n# TYPE stock_price gauge\n';
	metrics += results.map((r, i) => `stock_price{ticker="${environment.tickers[i]}"} ${r.c}`).join('\n');
	res.contentType('text/plain').send(metrics);
}));

// Error handling (must come last)
app.use(errorMiddleware);
process.on('unhandledRejection', (event) => errorMiddleware(<any>event));

// Startup
try {
	httpServer.listen(environment.port, async () =>
		console.log(`Listening on: http://localhost:${environment.port}`));
} catch(err) {
	console.error(err.message);
	process.exit(1);
}
